/*-
 * Copyright (c) 2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include "compat_glibc.h"
#include "compat_linux_utility.h"
#include <sys/signalvar.h>
#include <signal.h>


/*
 * Conversion of Linux sigset to BSD sigset
 */
__sym_default(linux_to_bsd_sigset, compat_linux_linux_to_bsd_sigset, COMPAT_LINUX_private);

void
compat_linux_linux_to_bsd_sigset(sigset_t *bsa_mask, const linux_sigset_t *lsa_mask)
{
	int b_sig, l_sig;

	SIGEMPTYSET(*bsa_mask);

	bsa_mask->__bits[0] = *lsa_mask & ~((1U << LINUX_SIGTBLSZ) - 1);
	bsa_mask->__bits[1] = 0;

	for(  l_sig = 1;  l_sig <= LINUX_SIGTBLSZ;  l_sig++  )  {
		if(  LINUX_SIGISMEMBER(*lsa_mask, l_sig)  )  {
			if(  (b_sig = lsig_to_bsig(l_sig))  )
				SIGADDSET(*bsa_mask, b_sig);
		}
	}
}
