/*-
 * Copyright (c) 2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

#include "compat_glibc.h"
#include "compat_linux_utility.h"
#include <signal.h>


/*
 * Conversion of Linux signals to BSD signals
 */
static int linux_to_bsd_signal[LINUX_SIGTBLSZ] = {
	SIGHUP,  SIGINT,    SIGQUIT, SIGILL,
	SIGTRAP, SIGABRT,   SIGBUS,  SIGFPE,
	SIGKILL, SIGUSR1,   SIGSEGV, SIGUSR2,
	SIGPIPE, SIGALRM,   SIGTERM, SIGBUS,
	SIGCHLD, SIGCONT,   SIGSTOP, SIGTSTP,
	SIGTTIN, SIGTTOU,   SIGURG,  SIGXCPU,
	SIGXFSZ, SIGVTALRM, SIGPROF, SIGWINCH,
	SIGIO,   SIGURG,    SIGSYS
};

__sym_default(lsig2bsig, compat_linux_lsig2bsig, COMPAT_LINUX_private);

int
compat_linux_lsig2bsig(const int lsig)
{
	return linux_to_bsd_signal[_SIG_IDX(lsig)];
}
