/*-
 * Copyright (c) 2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"

void abort(void);
int write(int, const void *, unsigned int);

#define hook_and_abort_2(func, major, minor)	\
	GLIBC_COMPAT_2(func, major, minor); \
	void GLIBC_SYMBOL_2(func, major, minor)(void) { \
		write(2, #func "\n", sizeof(#func "\n") - 1); \
		abort(); \
	}

#define hook_and_abort_3(func, major, minor, patch)	\
	GLIBC_COMPAT_3(func, major, minor, patch); \
	void GLIBC_SYMBOL_3(func, major, minor, patch)(void) { \
		write(2, #func "\n", sizeof(#func "\n") - 1); \
		abort(); \
	}

hook_and_abort_2(__assert_fail, 2, 0);
hook_and_abort_2(__ctype_b, 2, 0);
hook_and_abort_2(__ctype_toupper, 2, 0);
//hook_and_abort_2(__errno_location, 2, 0);
hook_and_abort_2(__fxstat, 2, 0);
hook_and_abort_2(__strtol_internal, 2, 0);
hook_and_abort_2(__strtoul_internal, 2, 0);
//hook_and_abort_2(__xstat, 2, 0);
//hook_and_abort_2(_exit, 2, 0);
hook_and_abort_2(_setjmp, 2, 0);
//hook_and_abort_2(abort, 2, 0);
//hook_and_abort_2(close, 2, 0);
//hook_and_abort_2(closedir, 2, 0);
//hook_and_abort_2(connect, 2, 0);
//hook_and_abort_2(dlclose, 2, 0);
//hook_and_abort_2(dlerror, 2, 0);
//hook_and_abort_2(dlsym, 2, 0);
//hook_and_abort_2(dup2, 2, 0);
//hook_and_abort_2(execlp, 2, 0);
//hook_and_abort_2(fcntl, 2, 0);
//hook_and_abort_2(feof, 2, 0);
//hook_and_abort_2(fgets, 2, 0);
//hook_and_abort_2(fork, 2, 0);
//hook_and_abort_2(fread, 2, 0);
//hook_and_abort_2(free, 2, 0);
//hook_and_abort_2(fseek, 2, 0);
//hook_and_abort_2(ftell, 2, 0);
//hook_and_abort_2(fwrite, 2, 0);
//hook_and_abort_2(getcwd, 2, 0);
//hook_and_abort_2(getenv, 2, 0);
hook_and_abort_2(gethostbyname, 2, 0);
//hook_and_abort_2(getpid, 2, 0);
//hook_and_abort_2(getsockopt, 2, 0);
//hook_and_abort_2(gettimeofday, 2, 0);
//hook_and_abort_2(gmtime, 2, 0);
//hook_and_abort_2(ioctl, 2, 0);
//hook_and_abort_2(localtime, 2, 0);
hook_and_abort_2(longjmp, 2, 0);
//hook_and_abort_2(malloc, 2, 0);
//hook_and_abort_2(memcpy, 2, 0);
//hook_and_abort_2(memmove, 2, 0);
//hook_and_abort_2(memset, 2, 0);
//hook_and_abort_2(mkdir, 2, 0);
//hook_and_abort_2(mktime, 2, 0);
hook_and_abort_2(mmap, 2, 0);
hook_and_abort_2(munmap, 2, 0);
//hook_and_abort_2(open, 2, 0);
//hook_and_abort_2(opendir, 2, 0);
//hook_and_abort_2(pipe, 2, 0);
//hook_and_abort_2(printf, 2, 0);
//hook_and_abort_2(qsort, 2, 0);
//hook_and_abort_2(rand, 2, 0);
//hook_and_abort_2(read, 2, 0);
//hook_and_abort_2(readdir, 2, 0);
//hook_and_abort_2(realloc, 2, 0);
hook_and_abort_2(recv, 2, 0);
//hook_and_abort_2(rename, 2, 0);
//hook_and_abort_2(rmdir, 2, 0);
//hook_and_abort_2(select, 2, 0);
//hook_and_abort_2(semget, 2, 0);
hook_and_abort_2(semop, 2, 0);
//hook_and_abort_2(setlocale, 2, 0);
//hook_and_abort_2(shmat, 2, 0);
//hook_and_abort_2(shmdt, 2, 0);
//hook_and_abort_2(shmget, 2, 0);
//hook_and_abort_2(socket, 2, 0);
//hook_and_abort_2(sprintf, 2, 0);
//hook_and_abort_2(srand, 2, 0);
//hook_and_abort_2(strchr, 2, 0);
//hook_and_abort_2(strcmp, 2, 0);
//hook_and_abort_2(strcpy, 2, 0);
//hook_and_abort_2(strdup, 2, 0);
//hook_and_abort_2(strncat, 2, 0);
//hook_and_abort_2(strncmp, 2, 0);
//hook_and_abort_2(strncpy, 2, 0);
//hook_and_abort_2(strstr, 2, 0);
//hook_and_abort_2(system, 2, 0);
//hook_and_abort_2(tempnam, 2, 0);
//hook_and_abort_2(time, 2, 0);
//hook_and_abort_2(uname, 2, 0);
//hook_and_abort_2(unlink, 2, 0);
//hook_and_abort_2(usleep, 2, 0);
//hook_and_abort_2(waitpid, 2, 0);
//hook_and_abort_2(write, 2, 0);
//hook_and_abort_2(dlopen, 2, 1);
//hook_and_abort_2(fclose, 2, 1);
//hook_and_abort_2(fopen, 2, 1);
//hook_and_abort_2(iconv, 2, 1);
//hook_and_abort_2(iconv_open, 2, 1);
//hook_and_abort_2(pclose, 2, 1);
//hook_and_abort_2(popen, 2, 1);
//hook_and_abort_3(__cxa_atexit, 2, 1, 3);
//hook_and_abort_3(__cxa_finalize, 2, 1, 3);
hook_and_abort_2(semctl, 2, 2);
//hook_and_abort_2(shmctl, 2, 2);
hook_and_abort_3(dl_iterate_phdr, 2, 2, 4);



hook_and_abort_2(_IO_getc, 2, 0);
hook_and_abort_2(__ctype_tolower, 2, 0);
hook_and_abort_2(__strdup, 2, 0);
//hook_and_abort_2(accept, 2, 0);
//hook_and_abort_2(alarm, 2, 0);
hook_and_abort_2(bind, 2, 0);
//hook_and_abort_2(calloc, 2, 0);
//hook_and_abort_2(execvp, 2, 0);
//hook_and_abort_2(fileno, 2, 0);
//hook_and_abort_2(fprintf, 2, 0);
//hook_and_abort_2(fputs, 2, 0);
hook_and_abort_2(getpwnam, 2, 0);
hook_and_abort_2(getpwuid, 2, 0);
//hook_and_abort_2(getuid, 2, 0);
//hook_and_abort_2(kill, 2, 0);
//hook_and_abort_2(listen, 2, 0);
//hook_and_abort_2(mkfifo, 2, 0);
//hook_and_abort_2(putenv, 2, 0);
//hook_and_abort_2(raise, 2, 0);
//hook_and_abort_2(setsockopt, 2, 0);
//hook_and_abort_2(sigaction, 2, 0);
hook_and_abort_2(sigemptyset, 2, 0);
//hook_and_abort_2(snprintf, 2, 0);
hook_and_abort_2(sscanf, 2, 0);
hook_and_abort_2(stderr, 2, 0);
hook_and_abort_2(stdin, 2, 0);
hook_and_abort_2(stdout, 2, 0);
//hook_and_abort_2(strcasecmp, 2, 0);
//hook_and_abort_2(strcat, 2, 0);
//hook_and_abort_2(strcoll, 2, 0);
hook_and_abort_2(strpbrk, 2, 0);
hook_and_abort_2(strtok, 2, 0);
//hook_and_abort_2(sysconf, 2, 0);
//hook_and_abort_2(tmpnam, 2, 0);
//hook_and_abort_2(tolower, 2, 0);
//hook_and_abort_2(toupper, 2, 0);
//hook_and_abort_2(ungetc, 2, 0);
//hook_and_abort_2(vfork, 2, 0);
hook_and_abort_2(vsnprintf, 2, 0);
//hook_and_abort_2(fdopen, 2, 1);
//hook_and_abort_2(tmpfile, 2, 1);
