/*-
 * Copyright (c) 2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"
#include <unistd.h>
#include <stdarg.h>

/*
 * execlp(3)
 */
GLIBC_COMPAT_2(execlp, 2, 0);

int
GLIBC_SYMBOL_2(execlp, 2, 0)
	(const char *name, const char *arg, ...)
{
	int n, ret;
	va_list ap;
	char **argv;

	CLEAR_ERRNO;

	va_start(ap, arg);
	for(  n = 1;  va_arg(ap, char *) != NULL;  n++  )
		;
	va_end(ap);

	argv = alloca((n + 1) * sizeof(*argv));
	if (argv == NULL) {
		errno = ENOMEM;
		return (-1);
	}

	argv[0] = (char *)arg;

	va_start(ap, arg);
	for(  n = 1;  (argv[n] = va_arg(ap, char *)) != NULL;  n++  )
                ;
        va_end(ap);

	ret = execvp(name, argv);

        dprintf("execlp('%s', '%s', ...) = %d / ERRNO = %d", name, arg, ret, errno);

        return ret;
}
