/*-
 * Copyright (c) 2002-2004,2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"
#include <fcntl.h>
#include <stdarg.h>


/*
 * open(2)
 */
GLIBC_COMPAT_2(open, 2, 0);

int
GLIBC_SYMBOL_2(open, 2, 0)
	(const char *path, int l_flags, .../* mode_t mode */)
{
	va_list ap;
	int ret, flags = 0;

	va_start(ap, l_flags);

	if(  l_flags & LINUX_O_RDONLY    )  flags |= O_RDONLY;
	if(  l_flags & LINUX_O_WRONLY    )  flags |= O_WRONLY;
	if(  l_flags & LINUX_O_RDWR      )  flags |= O_RDWR;
	if(  l_flags & LINUX_O_NDELAY    )  flags |= O_NONBLOCK;
	if(  l_flags & LINUX_O_APPEND    )  flags |= O_APPEND;
	if(  l_flags & LINUX_O_SYNC      )  flags |= O_FSYNC;
	if(  l_flags & LINUX_O_NONBLOCK  )  flags |= O_NONBLOCK;
	if(  l_flags & LINUX_FASYNC      )  flags |= O_ASYNC;
	if(  l_flags & LINUX_O_CREAT     )  flags |= O_CREAT;
	if(  l_flags & LINUX_O_TRUNC     )  flags |= O_TRUNC;
	if(  l_flags & LINUX_O_EXCL      )  flags |= O_EXCL;
	if(  l_flags & LINUX_O_NOCTTY    )  flags |= O_NOCTTY;

	CLEAR_ERRNO;

	if(  flags & O_CREAT  )  {
		ret = open(path, flags, va_arg(ap, mode_t *));
	}  else  {
		ret = open(path, flags);
	}

	dprintf("open('%s', (L0%06o => F0%06o), ...) = %d / ERRNO = %d", path, l_flags, flags, ret, errno);

	va_end(ap);

	return ret;
}
