/*-
 * Copyright (c) 2002-2005,2006 Norikatsu Shigemura
 * Copyright (c) 2006 MANTANI Nobutaka
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"
#include <stddef.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/limits.h>
#include <netinet/in.h>


/*
 * connect(2)
 */
GLIBC_COMPAT_2(connect, 2, 0);

int
GLIBC_SYMBOL_2(connect, 2, 0)
	(int s, const struct linux_sockaddr *l_name, int namelen)
{
	int ret, domain;
	struct sockaddr_storage name;

	CLEAR_ERRNO;

	if(  namelen < 2 || namelen > UCHAR_MAX || l_name == NULL  )  {
		errno = EINVAL;
		return -1;
	}

	memcpy(&name, l_name, namelen);

	if(  ( domain = linux_to_bsd_domain(l_name->sa_family) ) == -1  )  {
		errno = EINVAL;
		return -1;
	}  else  {
		name.ss_family = domain;
	}

	if(  name.ss_family == AF_INET  )  {
		name.ss_len = sizeof(struct sockaddr_in);
	}  else  {
		name.ss_len = namelen;
	}

	ret = connect(s, (const struct sockaddr*)&name, name.ss_len);

#ifdef	DEBUG
	if(  name.ss_family == AF_LOCAL  )  {
		char buf[UCHAR_MAX];
		strlcpy(buf, name.__ss_pad1, namelen - 2 + 1);
		dprintf("connect(s=%d, name=%s, namelen=%d(%d on linux) = %d / ERRNO = %d", s, buf, name.ss_len, namelen, ret, errno);
	}  else  {
		dprintf("connect(s=%d, name=0x%08x(0x%08x on linux), namelen=%d(%d on linux) = %d / ERRNO = %d", s, &name, l_name, name.ss_len, namelen, ret, errno);
	}
#endif

	return ret;
}
