/*-
 * Copyright (c) 2002-2005,2006 Norikatsu Shigemura
 * Copyright (c) 2006 MANTANI Nobutaka
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"
#include "compat_linux_utility.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>


/*
 * getsockopt(2)
 */
GLIBC_COMPAT_2(getsockopt, 2, 0);

int
GLIBC_SYMBOL_2(getsockopt, 2, 0)
	(int s, int l_level, int l_optname, void *optval, socklen_t *optlen)
{
	int ret, optname, level = linux_to_bsd_sockopt_level(l_level);

	CLEAR_ERRNO;

	switch(  level  ) {
	case SOL_SOCKET:
		optname = linux_to_bsd_so_sockopt(l_optname);
		break;

	case IPPROTO_IP:
		optname = linux_to_bsd_ip_sockopt(l_optname);
		break;

	case IPPROTO_TCP:
		/* Linux TCP option values match BSD's */
		optname = l_optname;
		break;

	default:
		optname = -1;
		break;
	}

	if(  optname == -1  )  {
		errno = EINVAL;
		ret   = -1;
	}  else  {
		ret = getsockopt(s, level, optname, optval, optlen);
	}

	dprintf("getsocketopt(s=%d, level=%d(%d on linux), optname=%d(%d on linux), optval=%0d, optlen=%d(result)) = %d / ERRNO = %d", s, level, l_level, optname, l_optname, *(int*)optval, *optlen, ret, errno);

	return ret;
}

