/*-
 * Copyright (c) 2002-2003,2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"
#include <sys/stat.h>
#include <string.h>


#define	_STAT_VER	3

static void
__stat_copyout__(struct linux_stat *buf, struct stat *sb)
{
#if 0
	dev_t dev;
#endif

	memset(buf, '\0', sizeof(struct linux_stat));

	buf->st_dev	= (sb->st_dev & 0xffff00ff) | (sb->st_dev & 0xff00);
	buf->st_ino	= sb->st_ino;
	buf->st_mode	= sb->st_mode;
	buf->st_nlink	= sb->st_nlink;
	buf->st_uid	= sb->st_uid;
	buf->st_gid	= sb->st_gid;
	buf->st_rdev	= sb->st_rdev;
	buf->st_size	= sb->st_size;
	buf->st_atime	= sb->st_atime;
	buf->st_mtime	= sb->st_mtime;
	buf->st_ctime	= sb->st_ctime;
	buf->st_blksize	= sb->st_blksize;
	buf->st_blocks	= sb->st_blocks;

	/* Lie about disk drives which are character devices
	 * in FreeBSD but block devices under Linux.
	 */
#if 0
	if(  S_ISCHR(buf->st_mode) &&
	     (dev = makedev((sb->st_rdev & 0xff00) >> 8, (sb->st_rdev & 0xffff00ff))) != NODEV  ) {
		cdevsw = devsw(dev);
		if (cdevsw != NULL && (cdevsw->d_flags & D_DISK)) {
			buf->st_mode &= ~S_IFMT;
			buf->st_mode |= S_IFBLK;

			/* XXX this may not be quite right */
			/* Map major number to 0 */
			buf->st_dev = uminor(sb->st_dev) & 0xf;
			buf->st_rdev = sb->st_rdev & 0xff;
		}
		dprintf("__stat_copyout__");
	}
#endif
}


/*
 * __xstat(2)
 */
GLIBC_COMPAT_2(__xstat, 2, 0);

int
GLIBC_SYMBOL_2(__xstat, 2, 0)
	(int vers, const char *file, struct linux_stat *buf)
{
	int ret;
	struct stat sb;

	CLEAR_ERRNO;

	if(  vers != _STAT_VER || file == NULL || buf == NULL  )  {
		errno = EINVAL;
		ret = -1;
	}  else  {
		if(  (ret = stat(file, &sb)) == 0  )
			__stat_copyout__(buf, &sb);
	}

	dprintf("__xstat(%d, '%s', [0x%p]) = %d / ERRNO = %d", vers, file, buf, ret, errno);

	return ret;
}
