/*-
 * Copyright (c) 2002-2004,2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "compat_glibc.h"
#include <fcntl.h>
#include <stdarg.h>


/*
 * fcntl(2)
 */
GLIBC_COMPAT_2(fcntl, 2, 0);

int
GLIBC_SYMBOL_2(fcntl, 2, 0)
	(int fd, int l_cmd, ...)
{
	va_list ap;
	int arg = 0, l_arg;
	int ret, l_ret = 0;

	va_start(ap, l_cmd);

	CLEAR_ERRNO;

	switch(  l_cmd  )  {
	case LINUX_F_GETFD:
		ret = fcntl(fd, F_GETFD);

		dprintf("fcntl(%d, LINUX_F_GETFD) = %d / ERRNO = %d", fd, ret, errno);

		break;
		
        case LINUX_F_SETFD:
		ret = fcntl(fd, F_SETFD, (l_arg = va_arg(ap, int)));

		dprintf("fcntl(%d, LINUX_F_SETFD, %d) = %d / ERRNO = %d", fd, l_arg, ret, errno);

		break;

	case LINUX_F_GETFL:
		ret = fcntl(fd, F_GETFL);

		if(  ret >= 0  )  {
			if(  ret & O_RDONLY  )  l_ret |= LINUX_O_RDONLY;
			if(  ret & O_WRONLY  )  l_ret |= LINUX_O_WRONLY;
			if(  ret & O_RDWR    )  l_ret |= LINUX_O_RDWR;
			if(  ret & O_NDELAY  )  l_ret |= LINUX_O_NONBLOCK;
			if(  ret & O_APPEND  )  l_ret |= LINUX_O_APPEND;
			if(  ret & O_FSYNC   )  l_ret |= LINUX_O_SYNC;
			if(  ret & O_ASYNC   )  l_ret |= LINUX_FASYNC;
		}  else  {
			l_ret = -1;
		}

		dprintf("fcntl(%d, LINUX_F_GETFL) = (L0%06o <= F0%06o) / ERRNO = %d", fd, l_ret, ret, errno);

		ret = l_ret;
		break;

        case LINUX_F_SETFL:
		l_arg = va_arg(ap, int);

		if(  l_arg & LINUX_O_NDELAY  )  arg |= O_NONBLOCK;
                if(  l_arg & LINUX_O_APPEND  )  arg |= O_APPEND;
                if(  l_arg & LINUX_O_SYNC    )  arg |= O_FSYNC;
                if(  l_arg & LINUX_FASYNC    )  arg |= O_ASYNC;

		ret = fcntl(fd, F_SETFL, arg);

		dprintf("fcntl(%d, LINUX_F_SETFL, (L0%06o => F0%06o)) = %d / ERRNO = %d", fd, l_arg, arg, ret, errno);

		break;

	default:
		errno = ENOSYS;
		ret = -1;

		dprintf("[not supported] fcntl(%d, %d, ...) = %d / ERRNO = %d", fd, l_cmd, ret, errno);

		break;
	}

	return ret;
}
