/*-
 * Copyright (c) 2004,2006 Norikatsu Shigemura
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

/*
 *
 */
#include "compat_glibc.h"
#include "compat_linux_utility.h"
#include <sys/stddef.h>
#include <sys/signalvar.h>
#include <signal.h>

/*
 * sigaction(2)
 */
GLIBC_COMPAT_2(sigaction, 2, 0);

int
GLIBC_SYMBOL_2(sigaction, 2, 0)
	(int l_sig, const struct linux_sigaction *lact, struct linux_sigaction *olact)
{
	int ret, sig = lsig2bsig(l_sig);
	struct sigaction act, oact;

	CLEAR_ERRNO;

	if(  l_sig <= 0  ||  LINUX_NSIG < l_sig  )  {
		errno = EINVAL;
		ret = -1;
	}  else  {
		if(  lact != NULL  )  {
			act.sa_handler = lact->lsa_handler;
			linux_to_bsd_sigset(&act.sa_mask, &lact->lsa_mask);
			act.sa_flags   = 0;
			if(  lact->lsa_flags & LINUX_SA_NOCLDSTOP  )
				act.sa_flags |= SA_NOCLDSTOP;
			if(  lact->lsa_flags & LINUX_SA_NOCLDWAIT  )
				act.sa_flags |= SA_NOCLDWAIT;
			if(  lact->lsa_flags & LINUX_SA_SIGINFO  )
				act.sa_flags |= SA_SIGINFO;
			if(  lact->lsa_flags & LINUX_SA_ONSTACK  )
				act.sa_flags |= SA_ONSTACK;
			if(  lact->lsa_flags & LINUX_SA_RESTART  )
				act.sa_flags |= SA_RESTART;
			if(  lact->lsa_flags & LINUX_SA_ONESHOT  )
				act.sa_flags |= SA_RESETHAND;
			if(  lact->lsa_flags & LINUX_SA_NOMASK  )
				act.sa_flags |= SA_NODEFER;
		}

		ret = sigaction(sig, (lact ? &act : NULL), &oact);

		if(  olact != NULL  )  {
			olact->lsa_handler  = oact.sa_handler;
			olact->lsa_restorer = NULL;
			bsd_to_linux_sigset(&olact->lsa_mask, &oact.sa_mask);
			olact->lsa_flags    = 0;
			if(  oact.sa_flags & SA_NOCLDSTOP  )
				olact->lsa_flags |= LINUX_SA_NOCLDSTOP;
			if(  oact.sa_flags & SA_NOCLDWAIT  )
				olact->lsa_flags |= LINUX_SA_NOCLDWAIT;
			if(  oact.sa_flags & SA_SIGINFO  )
				olact->lsa_flags |= LINUX_SA_SIGINFO;
			if(  oact.sa_flags & SA_ONSTACK  )
				olact->lsa_flags |= LINUX_SA_ONSTACK;
			if(  oact.sa_flags & SA_RESTART  )
				olact->lsa_flags |= LINUX_SA_RESTART;
			if(  oact.sa_flags & SA_RESETHAND  )
				olact->lsa_flags |= LINUX_SA_ONESHOT;
			if(  oact.sa_flags & SA_NODEFER  )
				olact->lsa_flags |= LINUX_SA_NOMASK;
		}
	}

	dprintf("sigaction(%d(%d on linux), [0x%p], [0x%p]) = %d / ERRNO = %d", sig, l_sig, lact, olact, ret, errno);

	return ret;
}
